# -*- coding: utf-8 -*-

# Copyright (c) Pilot Systems, 2013

# This file is part of Faceted SeSQL.

# SeSQL is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# SeSQL is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Faceted SeSQL.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.utils.translation import ugettext_lazy as _

CRUST_TYPES = ((1, _("normal")),
               (2, _("thin")),
               (3, _("large")),
               (4, _("with cheese")))

QUALITIES = ((1, 'halal'),
             (2, 'casher'),
             (3, 'vegetarian'),
             (4, 'vegan'))

QUALITY_NAMES = { 1: _('halal'),
                  2: _('casher'),
                  3: _('vegetarian'),
                  4: _('vegan') }

DAYS = { 1: _('monday'),
         2: _('tuesday'),
         3: _('wednesday'),
         4: _('thursday'),
         5: _('friday'),
         6: _('saturday'),
         7: _('sunday') }

class Shop(models.Model):
    name = models.CharField(max_length=100, unique=True)
    opendays = models.CharField(max_length=100, verbose_name = _('open days, enter like 1-3,5-6 (0=7=sunday)'))

    def __unicode__(self):
        return self.name

    def computed_opendays(self):
        """
        Compute days where the shop is open
        """
        days = [ False ] * 8
        for word in self.opendays.split(','):
            word = word.strip()
            if '-' in word:
                start, end = word.split('-')
            else:
                start, end = word, word
            start = int(start.strip())
            end = int(end.strip())
            for day in range(start, end + 1):
                days[(day % 7) or 7] = True
        return tuple([ d for d,v in enumerate(days) if v ])

    def openday_names(self):
        """
        Get openday names
        """
        return [ DAYS[day] for day in self.computed_opendays() ]

    def get_related_objects_for_indexation(self):
        """
        List what must be reindexed when this is modified
        """
        res = list(Pizza.objects.filter(shop = self))
        return res

class Topping(models.Model):
    name = models.CharField(max_length=100, unique=True)
    halal = models.BooleanField(default=False)
    casher = models.BooleanField(default=False)
    vegetarian = models.BooleanField(default=False)
    vegan = models.BooleanField(default=False)

    def __unicode__(self):
        return self.name

class Pizza(models.Model):
    name = models.CharField(max_length=100, unique=True)
    shop = models.ForeignKey(Shop, related_name='pizzas')
    persons = models.PositiveIntegerField(verbose_name = _('number of persons'))
    weight = models.PositiveIntegerField(verbose_name = _('weight (g)'))
    calories = models.PositiveIntegerField()
    price = models.PositiveIntegerField()
    crust = models.PositiveIntegerField(u"crust type", choices = CRUST_TYPES)
    toppings = models.ManyToManyField(Topping)

    def __unicode__(self):
        return self.name

    def is_all(self, what):
        """
        Check if all toppings share a quality
        """
        vals = [ getattr(t, what) for t in self.toppings.all() ]
        return min(vals) if vals else True

    @property
    def qualities(self):
        """
        Get the list of qualities we have, as integers
        """
        return tuple([ val for val, name in QUALITIES if self.is_all(name) ])

    @property
    def quality_names(self):
        return tuple([ QUALITY_NAMES[q] for q in self.qualities ])
